@@ -21,15 +21,14 @@
  * @const
  * @type {string}
  */
-const SERVER = (location.protocol + '//' + location.hostname +
-                (location.port ? ':' + location.port : ''));
+const SERVER = location.origin;
 
 /**
  * Path of the root collection on the server (must end with /)
  * @const
  * @type {string}
  */
-const ROOT_PATH = location.pathname.replace(new RegExp("/+[^/]+/*(/index\\.html?)?$"), "") + '/';
+const ROOT_PATH = (new URL("..", location.href)).pathname;
 
 /**
  * Regex to match and normalize color
@@ -120,7 +119,7 @@ function Collection(href, type, displayname, description, color) {
  */
 function get_principal(user, password, callback) {
     let request = new XMLHttpRequest();
-    request.open("PROPFIND", SERVER + ROOT_PATH, true, user, password);
+    request.open("PROPFIND", SERVER + ROOT_PATH, true, user, encodeURIComponent(password));
     request.onreadystatechange = function() {
         if (request.readyState !== 4) {
             return;
@@ -163,7 +162,7 @@ function get_principal(user, password, callback) {
  */
 function get_collections(user, password, collection, callback) {
     let request = new XMLHttpRequest();
-    request.open("PROPFIND", SERVER + collection.href, true, user, password);
+    request.open("PROPFIND", SERVER + collection.href, true, user, encodeURIComponent(password));
     request.setRequestHeader("depth", "1");
     request.onreadystatechange = function() {
         if (request.readyState !== 4) {
@@ -264,7 +263,7 @@ function get_collections(user, password, collection, callback) {
  */
 function upload_collection(user, password, collection_href, file, callback) {
     let request = new XMLHttpRequest();
-    request.open("PUT", SERVER + collection_href, true, user, password);
+    request.open("PUT", SERVER + collection_href, true, user, encodeURIComponent(password));
     request.onreadystatechange = function() {
         if (request.readyState !== 4) {
             return;
@@ -289,7 +288,7 @@ function upload_collection(user, password, collection_href, file, callback) {
  */
 function delete_collection(user, password, collection, callback) {
     let request = new XMLHttpRequest();
-    request.open("DELETE", SERVER + collection.href, true, user, password);
+    request.open("DELETE", SERVER + collection.href, true, user, encodeURIComponent(password));
     request.onreadystatechange = function() {
         if (request.readyState !== 4) {
             return;
@@ -314,7 +313,7 @@ function delete_collection(user, password, collection, callback) {
  */
 function create_edit_collection(user, password, collection, create, callback) {
     let request = new XMLHttpRequest();
-    request.open(create ? "MKCOL" : "PROPPATCH", SERVER + collection.href, true, user, password);
+    request.open(create ? "MKCOL" : "PROPPATCH", SERVER + collection.href, true, user, encodeURIComponent(password));
     request.onreadystatechange = function() {
         if (request.readyState !== 4) {
             return;
